/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core;

import javax.annotation.CheckForNull;
import net.java.truelicense.core.auth.Authentication;
import net.java.truelicense.core.auth.AuthenticationParameters;
import net.java.truelicense.core.crypto.Encryption;
import net.java.truelicense.core.io.Source;
import net.java.truelicense.core.util.Builder;
import net.java.truelicense.core.util.Injection;
import net.java.truelicense.obfuscate.*;

/**
 * A derived context for license vendor applications alias license key tools.
 * Use this context to configure a {@link LicenseVendorManager} with the
 * required parameters.
 * <p>
 * Here's an example of a licensing schema for vending MyApp 1.X license keys:
 * <pre><code>
public final class LicensingSchema {

    /** Returns the license vendor context. *{@literal}/
    public static LicenseVendorContext context() { return Lazy.context; }

    /** Returns the license vendor manager. *{@literal}/
    public static LicenseVendorManager manager() { return Lazy.manager; }

    private static class Lazy {

        &#064;Obfuscate
        static final String SUBJECT = "MyApp 1.X";

        &#064;Obfuscate
        static final String PRIVATE_KEY_STORE_NAME = "private.ks";

        static final ObfuscatedString PRIVATE_KEY_STORE_PASSWORD =
                new ObfuscatedString(new long[] {
                    0x17a9e5c922dc9155l, 0xef30516fcc5e98c2l }); /* => "test1234" *{@literal}/

        &#064;Obfuscate
        static final String PRIVATE_KEY_ENTRY_ALIAS = "mykey";

        static final ObfuscatedString PRIVATE_KEY_ENTRY_PASSWORD =
                new ObfuscatedString(new long[] {
                    0x149d045402a96977l, 0xa448f2162811f378l }); /* => "test1234" *{@literal}/

        static final ObfuscatedString PBE_PASSWORD =
                new ObfuscatedString(new long[] {
                    0xc9c00fc60c9db6b8l, 0x4a19910b12cd00c4l }); /* => "test1234" *{@literal}/

        static final LicenseVendorContext context;
        static final LicenseVendorManager manager;

        static {
            context = new V2LicenseManagementContext(SUBJECT).vendor();
            manager = context.manager(
                context.keyStore(
                    context.resource(PRIVATE_KEY_STORE_NAME),
                    null,
                    PRIVATE_KEY_STORE_PASSWORD,
                    PRIVATE_KEY_ENTRY_ALIAS,
                    PRIVATE_KEY_ENTRY_PASSWORD),
                context.pbe(null, PBE_PASSWORD));
        }
    }
}
 * </code></pre>
 * <p>
 * If you prefer a fluent API programming style, then you can substitute the
 * static initializer in the previous example with this:
 * <pre><code>
        static {
            context = new V2LicenseManagementContext(SUBJECT).vendor();
            manager = context
                .manager()
                    .keyStore()
                        .loadFromResource(PRIVATE_KEY_STORE_NAME)
                        .storePassword(PRIVATE_KEY_STORE_PASSWORD)
                        .alias(PRIVATE_KEY_ENTRY_ALIAS)
                        .keyPassword(PRIVATE_KEY_ENTRY_PASSWORD)
                        .inject()
                    .pbe()
                        .password(PBE_PASSWORD)
                        .inject()
                    .build();
        }
 * </code></pre>
 * <p>
 * Do not copy-paste this code!
 * Instead, use the TrueLicense Maven Archetype to generate a fully populated
 * template project for you.
 * <p>
 * Applications have no need to implement this interface and should not do so
 * because it may be subject to expansion in future versions.
 *
 * @author Christian Schlichtherle
 */
public interface LicenseVendorContext
extends LicenseApplicationContext, LicenseProvider {

    /**
     * Configures a license vendor manager.
     *
     * @param  authentication the authentication.
     * @param  encryption the encryption.
     * @return A license vendor manager.
     */
    LicenseVendorManager manager(Authentication authentication, Encryption encryption);

    /**
     * Configures a key store based authentication.
     * The provided strings should be computed on demand from an obfuscated form,
     * e.g. by annotating a constant string value with the &#64;{@link Obfuscate}
     * annotation and processing it with the TrueLicense Maven Plugin.
     * <p>
     * Applications should select a key store type with a strong encryption
     * algorithm for the private key entry, e.g. {@code JCEKS}.
     * If you choose a different key store type then you need to make sure that
     * it's supported by a security provider on all platforms, see
     * <a href="http://docs.oracle.com/javase/6/docs/technotes/guides/security/SunProviders.html">Java Cryptography Architeture Sun Providers Documentation</a>.
     *
     * @param  source the nullable source for the key store.
     *         May be {@code null} if and only if the key store type does not
     *         require loading from an input stream.
     * @param  storeType the nullable type of the key store.
     *         If this is {@code null}, then it defaults to a value as if
     *         computed by the expression
     *         <code>{@link #context()}.{@link LicenseManagementContext#storeType() storeType()}</code>.
     * @param  storePassword the password for the key store.
     * @param  alias the alias of the private key entry in the key store.
     * @param  keyPassword the password for the private key in the entry.
     * @return A key store based authentication.
     * @throws IllegalArgumentException if a password is considered to be too
     *         weak.
     */
    Authentication keyStore(@CheckForNull Source source, @CheckForNull String storeType, ObfuscatedString storePassword, String alias, ObfuscatedString keyPassword);

    /**
     * {@inheritDoc}
     *
     * @throws IllegalArgumentException if the password is considered to be too
     *         weak.
     */
    @Override Encryption pbe(String algorithm, ObfuscatedString password);

    /**
     * Returns a builder for
     * {@linkplain LicenseVendorManager license vendor managers}.
     * Call its {@link ManagerBuilder#build} method to build and return a
     * configured license vendor manager.
     * <p>
     * Use this method if you prefer a fluent API programming style.
     */
    ManagerBuilder manager();

    /**
     * A builder for a {@linkplain LicenseVendorManager license vendor manager}.
     * Call {@link #build} to obtain a configured license vendor manager.
     * <p>
     * Use this interface if you prefer a fluent API programming style.
     *
     * @author Christian Schlichtherle
     */
    interface ManagerBuilder extends Builder<LicenseVendorManager> {

        /**
         * Sets the authentication.
         * The authentication parameters need to be configured
         * {@linkplain AuthenticationParameters#forSigning for signing}.
         *
         * @return {@code this}.
         */
        ManagerBuilder authentication(Authentication authentication);

        /**
         * Returns an injection for a key store based authentication.
         * Call its {@link Injection#inject} method to build and inject the
         * configured authentication into this builder and return it.
         * <p>
         * The keystore needs to have a key password configured for the private
         * key entry.
         */
        KsbaInjection<ManagerBuilder> keyStore();

        /**
         * Sets the encryption.
         *
         * @return {@code this}.
         */
        ManagerBuilder encryption(Encryption encryption);

        /**
         * Returns an injection for a Password Based {@link Encryption} (PBE).
         * Call its {@link Injection#inject} method to build and inject the
         * configured encryption into this builder and return it.
         */
        PbeInjection<ManagerBuilder> pbe();
    }
}
